// ==============================================================
//                 ORBITER MODULE: module_p5
//
//
// module_p5.cpp
// Control module for module_p5 vessel class
//
//
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"

// ==============================================================
// Some vessel parameters
// ==============================================================

#define GREEN RGB(0, 255, 0)

const double MAX_GRAPPLING_DIST = 0.3;



class p5: public VESSEL2 {
public:

	p5 (OBJHANDLE hVessel, int flightmodel)
		: VESSEL2 (hVessel, flightmodel) {}

	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPostCreation();
	void clbkPreStep(double SimT, double SimDT, double MJD);
	void AutoGrapple (void);
	void ReleaseGrapple(void);
	void CheckChildAttach(void);
	void clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, HDC hDC);
	int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);

	ATTACHMENTHANDLE P5_attachment;
};




// ==============================================================
// p5 class interface
// ==============================================================

// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void p5::clbkSetClassCaps (FILEHANDLE cfg)
{


	// physical specs
	SetCW (0.4, 0.4, 0.4, 0.4);
	SetRotDrag (_V(1,1,1));
	if (GetFlightModel() >= 1) {
		SetPitchMomentScale (1e-3);
		SetBankMomentScale (1e-3);
	}
	SetTrimScale (0.1);
	SetPMI (_V(1.5, 2.5, 2));
	SetCrossSections (_V(6.5, 6.1, 7.2));

	SetMeshVisibilityMode ((AddMesh ("MBR_ITS/mbr_its5")), MESHVIS_ALWAYS);
	//SetCameraOffset (_V(0,2,-3.5));
	SetTouchdownPoints (_V(0,1,0), _V(-1,-1,0), _V(1,-1,0));

	P5_attachment  = CreateAttachment (false,_V( 1.46, 1.0451, 1.53),_V(0, 0, 1),_V(0,-1,0),"MP");


// reentry pstream
	PARTICLESTREAMSPEC rps = {
		0, 10, 40, 0, 0.001, 0.5, 4, 0.2, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_PSQRT, 0, 1, PARTICLESTREAMSPEC::ATM_PLOG, 6e7, 5e15
	};
	AddReentryStream (&rps);

}

// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new p5 (hvessel, flightmodel);
}
// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (p5*)vessel;
}
// ==============================================================
// Overloaded clbkPostCreation callback functions
// ==============================================================

void p5::clbkPostCreation ()
{


}


int p5::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate)
{
  if (!down) return 0; // only process keydown events

  if (KEYMOD_SHIFT (kstate)) {

  } else { // unmodified keys
    switch (key) {

		case OAPI_KEY_N: // grapple
		AutoGrapple();
		return 1;

		case OAPI_KEY_M: // release attachment
		ReleaseGrapple();
		return 1;

	}
  }
  return 0;
}


void p5::AutoGrapple (void)
{
	{
	             // grapple satellite

		VECTOR3 att_point_pos;
		VECTOR3 att_point_dir;
		VECTOR3 att_point_rot;
		GetAttachmentParams(P5_attachment, att_point_pos, att_point_dir, att_point_rot);
		VECTOR3 gpos, grms, pos, dir, rot;
		Local2Global (att_point_pos, grms);  // global position of grapple point

		// Search the complete vessel list for a grappling candidate.
		// Not very scalable ...
		for (DWORD i = 0; i < oapiGetVesselCount(); i++) {
			OBJHANDLE hV = oapiGetVesselByIndex (i);
			if (hV == GetHandle()) continue; // we don't want to grapple ourselves ...
			oapiGetGlobalPos (hV, &gpos);
			if (dist (gpos, grms) < (oapiGetSize (hV)+ 10)) { // in range
				VESSEL *v = oapiGetVesselInterface (hV);
				DWORD nAttach = v->AttachmentCount (true);
				for (DWORD j = 0; j < nAttach; j++) { // now scan all attachment points of the candidate
					ATTACHMENTHANDLE hAtt = v->GetAttachmentHandle (true, j);
					const char *id = v->GetAttachmentId (hAtt);
					if (strncmp (id, "MM", 2)) continue; // attachment point not compatible
					v->GetAttachmentParams (hAtt, pos, dir, rot);
					v->Local2Global (pos, gpos);
					if (dist (gpos, grms) < MAX_GRAPPLING_DIST) { // found one!

/// start release child
						for (DWORD k = 0; k < oapiGetVesselCount(); k++)
							{
							OBJHANDLE hVp = oapiGetVesselByIndex (k);
							if (hVp == GetHandle()) continue;
							if (hVp == hV) continue;
							VESSEL *w = oapiGetVesselInterface (hVp);
							DWORD nAttachp = w->AttachmentCount (false);
							for (DWORD l = 0; l < nAttachp; l++)
								{
								ATTACHMENTHANDLE hAttp = w->GetAttachmentHandle (false, l);
								if (hV == w->GetAttachmentStatus (hAttp))DetachChild (hAttp);
								}
							}
/// end of release child
						AttachChild (hV, P5_attachment, hAtt);

						return;
					}
				}
			}
		}

	}
}

void p5::ReleaseGrapple (void)
{
DetachChild (P5_attachment);
}
void p5::CheckChildAttach(void)
{
	OBJHANDLE hC = GetAttachmentStatus(P5_attachment);
	VESSEL *v = oapiGetVesselInterface (hC);
	DWORD nAttach = v->AttachmentCount (true);
	for (DWORD j = 0; j < nAttach; j++)
		{ // now scan all attachment points of the candidate
		ATTACHMENTHANDLE hAtt = v->GetAttachmentHandle (true, j);
		const char *id = v->GetAttachmentId (hAtt);
		if (strncmp (id, "GS", 2))continue;
		if(v->GetAttachmentStatus(hAtt))DetachChild (P5_attachment);
		return;
		}
}


void p5::clbkPreStep(double SimT, double SimDT, double MJD) {

	if (GetAttachmentStatus(P5_attachment)) CheckChildAttach();
}

void p5::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, HDC hDC)
{
	// draw the default HUD
	VESSEL2::clbkDrawHUD (mode, hps, hDC);

	double s = ((hps->H));
	double d = (s/768);


	//int wc = (int)(148*d);
	//int w = (int)(164*d);
	//int w1 = (int) (180*d);
	//int w2 = (int)(196*d);
	//int w3 = (int)(212*d);
	//int w4 = (int)(228*d);
	int w5 = (int)(244*d);
	//int w6 = (int)(260*d);
	//int w7 = (int)(276*d);
	//int w8 = (int)(292*d);
	//int w9 = (int)(308*d);
	//int w10 = (int)(324*d);
	//int w11 = (int)(340*d);



	/////////////////// attachment

	if (GetAttachmentStatus (P5_attachment))
	{
	SetTextColor(hDC,GREEN);
	char abuf[256];
	sprintf(abuf,"M = Release Parent Attachment Point");
	TextOut(hDC,10,w5,abuf,strlen(abuf));
	}
	else
	{
	SetTextColor(hDC,GREEN);
	char abuf[256];
	sprintf(abuf,"N = Grapple Parent Attachment Point");
	TextOut(hDC,10,w5,abuf,strlen(abuf));
	}

}
